@@ -14,3 +14,9 @@ from cbc_sdk.platform.processes import (Process, ProcessFacet,
 
 
 from cbc_sdk.platform.reputation import ReputationOverride
+
+from cbc_sdk.platform.grants import Grant
+
+from cbc_sdk.platform.users import User
+
+from cbc_sdk.platform.vulnerability_assessment import Vulnerability
